class ActionSerializersMixin:
    """Mixin for ViewSets that allows to have a dict with a serializer for
    each action.
    """

    action_serializers = {}

    def get_serializer_class(self):
        if hasattr(self, "action_serializers"):
            if self.action in self.action_serializers:
                return self.action_serializers[self.action]
        return super().get_serializer_class()



from django.db import models

from belt.models import StatusMixin
from tests.app.constants import STATUS_OPTIONS, DRAFT, PUBLISHED
from tests.app.managers import PostQuerySet, CategoryQuerySet


class Post(StatusMixin, models.Model):
    title = models.CharField(max_length=250)
    content = models.TextField()
    status = models.CharField(max_length=16, choices=STATUS_OPTIONS, default=DRAFT)

    objects = PostQuerySet.as_manager()

    SEARCH_FIELDS = ["title", "content"]
    ALLOWED_TRANSITIONS = [(DRAFT, PUBLISHED)]
    FORBIDDEN_TRANSITIONS = [(PUBLISHED, DRAFT)]

    TRANSITION_HANDLERS = {(DRAFT, PUBLISHED): "published"}

    def published(self):
        pass

    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs)


class Category(models.Model):
    objects = CategoryQuerySet.as_manager()



from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _


class StatusMixin(models.Model):
    """Mixin to handle status changes"""

    STATUS_FIELD = "status"
    ALLOWED_TRANSITIONS = []
    FORBIDDEN_TRANSITIONS = []
    TRANSITION_HANDLERS = {}

    class Meta:
        abstract = True

    def refresh_from_db(self, using=None, fields=None):
        super().refresh_from_db(using=using, fields=fields)
        if hasattr(self, "_transition"):
            delattr(self, "_transition")

    def get_status_transition(self):
        """Get status transition."""
        if self.pk:
            if hasattr(self, "_transition"):
                return self._transition
            previous = self._meta.model.objects.get(pk=self.pk)
            if previous.status != getattr(self, self.STATUS_FIELD):
                self._transition = previous.status, getattr(self, self.STATUS_FIELD)
                return self._transition

    def validate_transition(self):
        """Validates the transition."""
        transition = self.get_status_transition()
        # Check the transition is not a allowed transitions
        if (
            transition
            and self.ALLOWED_TRANSITIONS
            and transition not in self.ALLOWED_TRANSITIONS
        ):
            raise ValidationError(
                _(
                    f"A {self._meta.model_name} can't change from "
                    f"{transition[0]} to {transition[1]}"
                )
            )
        # Check the transition is not a forbidden transitions
        if (
            transition
            and self.FORBIDDEN_TRANSITIONS
            and transition in self.FORBIDDEN_TRANSITIONS
        ):
            raise ValidationError(
                _(
                    f"A {self._meta.model_name} can't change from "
                    f"{transition[0]} to {transition[1]}"
                )
            )

    def pre_status_handler(self, transition):
        """Method used to execute code before the status handler is called."""
        pass

    def post_status_handler(self, transition):
        """Method used to execute code after the status handler is called."""
        pass

    def get_transition_handler(self):
        """Get the transition handler between status."""
        transition = self.get_status_transition()
        if transition:
            handler_name = self.TRANSITION_HANDLERS.get(transition, "")
            return getattr(self, handler_name) if hasattr(self, handler_name) else None

    def clean(self):
        """Validates transition in clean."""
        self.validate_transition()

    def save(self, *args, **kwargs):
        # Gets the handler before the save
        transition = self.get_status_transition()
        transition_handler = self.get_transition_handler()
        super().save(*args, **kwargs)
        # Executes the handler after the save
        if transition_handler:
            self.pre_status_handler(transition)
            transition_handler()
            self.post_status_handler(transition)



DEBUG = True
USE_TZ = True

SECRET_KEY = "dummy"

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "belt",
    "tests.app",
]

SITE_ID = 1

MIDDLEWARE = ()



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['belt', 'belt.django_filters', 'belt.rest_framework']

package_data = \
{'': ['*']}

install_requires = \
['django', 'django-model-utils', 'djangorestframework', 'django-filter']

extras_require = \
{'doc': ['belt'],
 'test': ['pytest', 'pytest-django', 'pytest-cov', 'factory_boy']}

setup(name='django-belt',
      version='1.1.0',
      description='Django app made to get together a set of utilities to use with Django projects.',
      author='Marcos Gabarda',
      author_email='hey@marcosgabarda.com',
      url='https://github.com/marcosgabarda/django-belt',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
